'*********************************************************************
'*   This FILEOUT module has all the code for file I/O, along with the
'*   relevant error handling routine.
'*
'*
'*********************************************************************
DECLARE SUB SetFileType ()
DECLARE FUNCTION Fyle% ()
DECLARE FUNCTION SSPType% ()
DECLARE SUB ClearDataCompPrompted ()
DECLARE SUB CloseOList ()
DECLARE SUB RestoreScreen ()
DECLARE SUB Message (Text$)
DECLARE SUB DiskError ()
DECLARE SUB AbortErr (Error$)
DECLARE SUB StartOList ()
DECLARE FUNCTION OListDone$ ()
DECLARE FUNCTION NewFileQuery% ()
DECLARE SUB WriteFastHeader ()
DECLARE SUB Prompt (PromptLine$, Result$)
DECLARE FUNCTION OutPutMode! ()
DECLARE SUB CloseFile ()
DECLARE FUNCTION IsFileName$ (Name$, Ext$)
DECLARE FUNCTION OListClosed% ()
DECLARE FUNCTION DumbDate$ ()
DIM SHARED IsAppend%, IsBadName%, CanNotWrite%, FileType%, FileName$


TYPE FastCount
     Num       AS INTEGER
     ITime     AS STRING * 12
     Counts    AS INTEGER
END TYPE

TYPE DataLine
     IDate     AS STRING * 10
     ITime     AS STRING * 8
     Catalog   AS STRING * 1
     Object    AS STRING * 10
     Filter    AS STRING * 2
     CurInteg  AS INTEGER
     C1        AS LONG
     C2        AS LONG
     C3        AS LONG
     C4        AS LONG
     Integ     AS INTEGER
     Gain      AS INTEGER
     Comments  AS STRING * 20
END TYPE

DIM SHARED OutMode AS INTEGER, NewFile AS INTEGER, OListOpen$
IsAppend% = 1
OListOpen$ = "N"

DiskErrorHandler:
     CALL DiskError
     RESUME

OListError:
     IF ERR = 62 THEN
          OListOver% = 1
          RESUME NEXT
     ELSE
          CALL DiskError
          RESUME
     END IF

FileDoesNotExist:
     IsAppend% = 0
     RESUME NEXT

CouldNotWrite:
     CanNotWrite% = 1
     RESUME NEXT

NoOList:
     CALL Message("That OList does not exist!")
     IsBadName% = 1
     RESUME NEXT
'

'*********************************************************************
'*   CloseFile sets the OutMode to 0 (No file open) and then closes
'*   the current file.
'*
'*
'*********************************************************************
'
SUB CloseFile

     ON ERROR GOTO DiskErrorHandler:
    
     OutMode = 0
     CLOSE #2
     
     Message ("File " + FileName$ + " closed.")
     ON ERROR GOTO 0

     CALL CloseOList
     FileType% = 0
     CALL SetFileType
     CALL ClearDataCompPrompted

END SUB '

'*********************************************************************
'*   This routine simply sets the OListOpen$ variable to "N" and sends
'*   a message to the user.
'*
'*
'*********************************************************************
'
SUB CloseOList

     CLOSE #3
     OListOpen$ = "N"
     'CALL Message("OList closed.  Count will stop after this object.")

END SUB

'*********************************************************************
'*   DiskError handles any file I/O error by calling the AbortErr function
'*   with an appropriate error description.  This was lifted almost directly
'*   from the neph code.
'*   NOTE: A lot of robustness could be easily added here.
'*********************************************************************
'
SUB DiskError
     LastErrCode% = 0
     LastErrCode% = ERR
     SELECT CASE LastErrCode%
          CASE 4
               CALL AbortErr("Out of data error in input file.")
          CASE 52
               CALL AbortErr("Bad file name or number.")
          CASE 53
               CALL AbortErr("File not found.")
          CASE 55
               CALL AbortErr("File already open.")
          CASE 57
               CALL AbortErr("Device I/O error.")
          CASE 58
               CALL AbortErr("File already exists.")
          CASE 61
               CALL AbortErr("Disk full.")
          CASE 62
               CALL AbortErr("Input past end of file")
          CASE 64
               CALL AbortErr("Bad file name")
          CASE 67
               CALL AbortErr("Too many files")
          CASE 70
               CALL AbortErr("Unable to open non-sharable network file.")
          CASE 71
               CALL AbortErr("Drive not ready.")
          CASE 72
               CALL AbortErr("Disk-media error")
          CASE 75
               CALL AbortErr("Path/File access error.")
          CASE 76
               CALL AbortErr("Path not found.")
          CASE ELSE
               CALL Message("File does not exist!")
               'CALL AbortErr("Disk I/O error.  Error code =" + STR$(LastErrorCode%))
    END SELECT


END SUB '

FUNCTION Fyle%

     Fyle% = FileType%

END FUNCTION

'*********************************************************************
'*   This routine reads in next object (with catalog and filter) from
'*   the currently opened OList.  It'll just bomb if it doesn't have
'*   an open OList (but it shouldn't be called that way).
'*
'*********************************************************************
'
SUB GetNextObject (Object$, Cat$, Filt$)

    
     ON ERROR GOTO OListError:
     temp$ = INPUT$(3, #3)
    
     Cat$ = UCASE$(MID$(temp$, 2, 1))
     Object$ = INPUT$(10, #3)

     INPUT #3, Filt$

     Filt$ = RIGHT$(Filt$, 1)

     ON ERROR GOTO 0
             
END SUB '

'**************************************************************************
'*   This routine turns any string variable into a valid filename with a
'*   specified extension.
'*
'*
'**************************************************************************
'
FUNCTION IsFileName$ (Name$, Ext$)

     temp$ = ""
     Good = 1
     IsExt% = 0
     OtherDir = 0

     FOR I = 1 TO LEN(Name$)
         
          c$ = MID$(Name$, I, 1)
         
          IF c$ = ":" THEN OtherDir = 1
          IF c$ = " " THEN Good = 0
          IF c$ = "\" THEN OtherDir = 1
          IF c$ = "*" THEN Good = 0
          IF c$ = "?" THEN Good = 0
          IF c$ = ";" THEN Good = 0

          IF Good = 1 THEN
               temp$ = temp$ + c$
               length = length + 1
          END IF
         
          Good = 1
    
     NEXT I
    
     IF length = 0 THEN temp$ = "DEFAULT"
    
     IF INSTR(temp$, ".") > 1 THEN
          IsFileName$ = UCASE$(temp$)
          EXIT FUNCTION
     END IF

     IF INSTR(temp$, ".") = 1 THEN
          IsFileName$ = "DEFAULT" + LEFT$(UCASE$(temp$), 4)
     END IF

     IF OtherDir = 0 THEN
          IF length > 8 THEN temp$ = LEFT$(temp$, 8)
     END IF
     IsFileName$ = UCASE$(temp$) + "." + UCASE$(Ext$)

END FUNCTION

'*********************************************************************
'*   This function merely indicates if anything has been written to
'*   the current output file yet.
'*
'*
'*********************************************************************
'
FUNCTION NewFileQuery%

     NewFileQuery% = NewFile

END FUNCTION '

'*************************************************************************
'*   This is the wrapper function for the OListOpen$ variable.  It returns
'*   a 1 if the there is no OList open.
'*
'*
'*************************************************************************
'
FUNCTION OListClosed%

     IF OListOpen$ <> "Y" THEN OListClosed% = 1

END FUNCTION

'*********************************************************************
'*   This function returns a TRUE value if the current OList is all
'*   read out, or a FALSE if it hasn't.
'*
'*   Unpredictable if no OList opened (so don't do it).
'*********************************************************************
'
FUNCTION OListDone$

     ON ERROR GOTO DiskErrorHandler:
    
     IF EOF(3) THEN OListDone$ = "Y" ELSE OListDone$ = "N"

     ON ERROR GOTO 0

END FUNCTION '

'*********************************************************************
'*   This routine prompts for a new file name, and then attempts to
'*   open it.  Note the use of the ON ERRORs around the Prompt - Prompt
'*   can call the SSP interface which resets the error traps.
'*
'*   Update:  This routine also checks to see if the file exists before
'*   opening.  If it does, it asks the user whether he wants to replace
'*   or append the file.
'*********************************************************************
'
SUB OpenFile
   
     ON ERROR GOTO DiskErrorHandler:
    
     IF OutMode <> 0 THEN
          CALL CloseFile
     END IF

     NewFile = 1
    
     ON ERROR GOTO 0

     CALL Prompt("Enter output file name:  ", temp$)

     ON ERROR GOTO DiskErrorHandler:

     IsAppend% = 1

     FileName$ = IsFileName$(temp$, "RAW")
    
     ON ERROR GOTO FileDoesNotExist:        'this tests whether the file exists
     OPEN FileName$ FOR INPUT AS #2
     CLOSE #2
    
     IF IsAppend% = 0 THEN OPEN FileName$ FOR OUTPUT AS #2
    
     IF IsAppend% = 1 THEN
          CALL Prompt("File " + FileName$ + " exists.  [A]ppend / [r]eplace?  ", temp$)
         
          IF UCASE$(LEFT$(temp$, 1)) = "R" THEN
               OPEN FileName$ FOR OUTPUT AS #2
               IsAppend% = 0
         
          ELSE
               OPEN FileName$ FOR APPEND AS #2
               NewFile = 0
          END IF
    
     END IF

     ON ERROR GOTO CouldNotWrite:
          PRINT #2, "@" + CHR$(8);
          CLOSE #2
          IF IsAppend% THEN OPEN FileName$ FOR APPEND AS #2 ELSE OPEN FileName$ FOR OUTPUT AS #2
     ON ERROR GOTO 0

     IF CanNotWrite% = 1 THEN
          CALL Message("File " + FileName$ + " cannot be written to!")
          CanNotWrite% = 0
          EXIT SUB
     END IF

     CALL Message("File " + FileName$ + " opened.")
    
     OutMode = 1

     ON ERROR GOTO 0

     FileType% = 0
     CALL SetFileType

END SUB '

'*********************************************************************
'*   This attempts to open an OList.  If the file is opened, it then
'*   reads enough blank lines to get things ready to read in the first
'*   object, catalog, filter set.
'*
'*********************************************************************
'
SUB OpenOList
   
     IsBadName% = 0

     IF OListOpen$ = "Y" THEN
          Message ("An OList is already open!")
          EXIT SUB
     END IF
    
     CALL Prompt("Enter OList name: ", temp$)
     temp$ = IsFileName$(temp$, "OLS")

     ON ERROR GOTO NoOList:
     OPEN temp$ FOR INPUT AS #3
     IF IsBadName% THEN EXIT SUB

     Message ("OList " + temp$ + " opened.")
   
     ON ERROR GOTO DiskErrorHandler:
     INPUT #3, temp$
     INPUT #3, temp$
    
     OListOpen$ = "Y"
     OutMode = 2

     ON ERROR GOTO 0

END SUB

'*********************************************************************
'*   This is a wrapper function for the OutMode, used both in the
'*   menu activator and the SetCount functions.
'*
'*
'*********************************************************************
'
FUNCTION OutPutMode

     OutPutMode = OutMode
     IF IsTestRead% THEN OutPutMode = 0

END FUNCTION '

'*********************************************************************
'*   This function takes care of any file activaty needed to enter
'*   fastmode outputs.  Not much right now.
'*
'*
'*********************************************************************
'
SUB SetFastMode

     OutMode = 3

END SUB '

'*********************************************************************
'*   Same as the StartFastMode thing.  First checks to make sure an
'*   OList is open.
'*
'*
'*********************************************************************
'
SUB StartOList

     IF OListOpen$ = "Y" THEN OutMode = 2 ELSE CALL Message("You must open an OList first!")

END SUB '

'*********************************************************************
'*   This writes the header for a fast mode file.  It could be modified
'*   to make more use of the comment line.
'*
'*
'*********************************************************************
'
SUB WriteFastHeader

     ON ERROR GOTO DiskErrorHandler:
    
     IF IsAppend% = 1 THEN EXIT SUB

     SELECT CASE SSPType%
          CASE 1
               temp1$ = "SSP3"
          CASE 2
               temp1$ = "SSP3A"
          CASE 3
               temp1$ = "SSP5"
          CASE 4
               temp1$ = "SSP5A"
     END SELECT
     PRINT #2, "  FILENAME="; FileName$; "     FASTMODE OUTPUT FILE FROM " + CHR$(34) + temp1$ + CHR$(34)
     PRINT #2, "FILE WRITTEN ON "; DATE$
     PRINT #2, ""
     PRINT #2, ""
     PRINT #2, "               UT      COUNTS"

     ON ERROR GOTO 0

     FileType% = 1
     CALL SetFileType

END SUB '

'*********************************************************************
'*   This function writes a single line of fastmode data in a form
'*   readable by RPLOT.
'*
'*
'*********************************************************************
'
SUB WriteFastMode (ToWrite AS FastCount)

     ON ERROR GOTO DiskErrorHandler:
    
     IF NewFile = 1 THEN
          CALL WriteFastHeader
          NewFile = 0
     END IF

     PRINT #2, USING "   #####  &  #####"; ToWrite.Num; ToWrite.ITime; ToWrite.Counts

     ON ERROR GOTO 0

END SUB '

'*********************************************************************
'*   This routine writes the full slow mode header.  This one makes
'*   use of most of the comment space and seems to be fully compatible
'*   with the RPHOT package.
'*
'*********************************************************************
'
SUB WriteSlowHeader (TeleName$, Obsv$, Comment$)

     ON ERROR GOTO DiskErrorHandler:
        
     IF IsAppend% = 1 THEN EXIT SUB
     NewFile = 0
    
     SELECT CASE SSPType%
          CASE 1
               temp1$ = "SSP3"
          CASE 2
               temp1$ = "SSP3A"
          CASE 3
               temp1$ = "SSP5"
          CASE 4
               temp1$ = "SSP5A"
     END SELECT

     PRINT #2, " FILENAME="; UCASE$(FileName$);
     PRINT #2, SPACE$(14);
     PRINT #2, "RAW OUTPUT DATA FILE FROM " + CHR$(34) + temp1$ + CHR$(34)
     PRINT #2, " UT DATE= "; DumbDate$;
     PRINT #2, "   TELESCOPE= "; UCASE$(LEFT$(TeleName$ + SPACE$(16), 16));
     PRINT #2, " OBSERVER= "; UCASE$(Obsv$)
     PRINT #2, " CONDITIONS="; UCASE$(Comment$)
     PRINT #2, " MO-DY-YEAR    UT    CAT  OBJECT     F  ----------COUNTS---------- INT SCLE COMMENTS"

     ON ERROR GOTO 0

     FileType% = 2
     CALL SetFileType
     CLOSE #2

END SUB '

'*********************************************************************
'*   This writes a single line of SlowMode data in an RPHOT readable
'*   form.  I tried to limit myself to one piece of data per line.
'*
'*
'*********************************************************************
'
SUB WriteSlowLine (DL AS DataLine)

     ON ERROR GOTO DiskErrorHandler:
   
     OPEN FileName$ FOR APPEND AS #2
     PRINT #2, " ";
     PRINT #2, DL.IDate; " ";
     PRINT #2, DL.ITime; " ";
     PRINT #2, UCASE$(DL.Catalog); "    ";
     PRINT #2, UCASE$(DL.Object);
     PRINT #2, DL.Filter;
     PRINT #2, USING "#######"; DL.C1;
     PRINT #2, USING "#######"; DL.C2;
     PRINT #2, USING "#######"; DL.C3;
     PRINT #2, USING "#######"; DL.C4;
     PRINT #2, USING "#### "; DL.Integ / 1000;
     TGain = DL.Gain
     StrGain$ = RIGHT$(STR$(TGain), LEN(STR$(TGain)) - 1)
     PRINT #2, LEFT$(StrGain$ + "    ", 4) + " ";
  
     IF LEN(DL.Comments) <> 0 THEN
          PRINT #2, LEFT$(DL.Comments, 54)
     END IF
     DL.Comments = ""
     CLOSE #2
   
     ON ERROR GOTO 0

END SUB '

